#========================================================================================
# package      DE_EPAGES::AmazonPayments::t::AmazonRestClient
# base         DE_EPAGES::Database::Shop::Test
# state        private
#========================================================================================
package DE_EPAGES::AmazonPayments::t::AmazonRestClient;
use base DE_EPAGES::Database::API::Test;
use strict;
use Test::Exception;
use Test::Mock::LWP::Dispatch;
use Test::More;
use DE_EPAGES::AmazonPayments::API::Object::AmazonRestClient;
use DE_EPAGES::EPagesJ::API::Object::HTTPClient;
use DE_EPAGES::Test::Mock::EpagesObject;
use HTTP::Response;
use HTTP::Headers;
use HTTP::Status qw ( :constants status_message );
use DE_EPAGES::AmazonPayments::API::Constants qw (
    CREDENTIALS_OK
    AMAZON_SANDBOX_MODE
    ORDER_DRAFT
);

sub mockResponse {
    my ($Status, $Payload) = @_;

    my $Header = HTTP::Headers->new( 'Content-Type' => 'application/json; charset=UTF-8', 'Conection' => 'close' );
    my $Response = HTTP::Response->new( $Status, status_message($Status), $Header, $Payload );

    return $Response;
}

sub test {
    my $self = shift;

    $self->testCheck();
    $self->testGetAndSetOrderDetails();
    $self->testConfirmOrder();
    $self->testCancelOrder();
    $self->testCloseOrder();
    $self->testAuthorize();
    $self->testGetAuthorizationDetails();
    $self->testCloseAuthorization();
    $self->testCapture();
    $self->testGetCaptureDetails();
    done_testing();
}

sub testCheck {
    my $self = shift;

    my $HttpClient = Test::MockObject::Extends->new(
        DE_EPAGES::EPagesJ::API::Object::HTTPClient->new
    );
    $HttpClient->mock('get', sub {
        my $self = shift;
        my ($Url, $headers, $body) = @_;
        like($Url, qr{http://baseurl/rs/payments/amazon/check}, 'url');
        like($Url, qr{region=DE}, 'region query param');
        like($Url, qr{env=SANDBOX}, 'env query param');
        is_deeply($headers,{
            'X-AccessKeyId'      => 'valid_access_key',
            'X-AccessKeySecret'  => 'valid_secret_key',
            'X-MerchantId'       => 'valid_merchant_id',
        }, 'headers');
        return mockResponse(HTTP_OK, '{ "status": "OK" }');
    });

    my $Client = $self->restClient(
        Region => 'DE',
        hKeySet => {
            AccessKeyId => 'valid_access_key',
            SecretKey   => 'valid_secret_key',
            MerchantID  => 'valid_merchant_id',
        },
        HttpClient => $HttpClient
    );
    my $ResponseStatus = $Client->check();
    is($ResponseStatus, CREDENTIALS_OK, 'status: OK');
}

sub testGetAndSetOrderDetails {
    my $self = shift;

    my $body = sub { return sprintf('{"status": { "state": "%s" }, "constraints":["ShippingAddressNotSet"] }', shift); };

    $mock_ua->unmap_all;
    $mock_ua->map(qr{/rs/payments/amazon/orders/refAbc}, sub { mockResponse(HTTP_OK, $body->('Draft')); });

    my $Client = $self->restClient;
    my $Response = $Client->getOrderDetails('refAbc');
    is($Response->transStatus, ORDER_DRAFT, 'getOrderDetails: status');
    is_deeply($Response->constraints, ['ShippingAddressNotSet'], 'getOrderDetails: constraints'),

    $Response = $Client->setOrderDetails('refAbc', {});
    is($Response, ORDER_DRAFT, 'setOrderDetails: status');
}

sub testConfirmOrder {
    my $self = shift;

    $mock_ua->unmap_all;
    $mock_ua->map(qr{/rs/payments/amazon/orders/refAbc/confirmation}, sub { mockResponse(HTTP_NO_CONTENT, undef); });
    $mock_ua->map(qr{/rs/payments/amazon/orders/refConflict/confirmation}, sub { mockResponse(HTTP_BAD_REQUEST, undef); });

    my $Client = $self->restClient;

    # expect to be ok.
    $Client->confirmOrder('refAbc');
    pass('successful confirmation');
    throws_ok(sub { $Client->confirmOrder('refConflict') }, qr/REQUEST_FAILED.*400/, 'failed confirmation: BadRequest');
}

sub testCloseOrder {
    my $self = shift;

    $mock_ua->unmap_all;
    $mock_ua->map(qr{/rs/payments/amazon/orders/refId/close}, sub { mockResponse(HTTP_NO_CONTENT, undef); });
    my $Client = $self->restClient();

    # expect to be ok.
    # $authId, $captureRefId, $currency, $amount
    $Client->closeOrder('refId');
    pass('closeOrder ok');
}

sub testCancelOrder {
    my $self = shift;

    $mock_ua->unmap_all;
    $mock_ua->map(qr{/rs/payments/amazon/orders/refId/cancel}, sub { mockResponse(HTTP_NO_CONTENT, undef); });
    my $Client = $self->restClient();

    # expect to be ok.
    # $authId, $captureRefId, $currency, $amount
    $Client->cancelOrder('refId');
    pass('cancelOrder ok');
}

sub testAuthorize {
    my $self = shift;

    $mock_ua->unmap_all;
    $mock_ua->map(qr{/rs/payments/amazon/authorizations[^/]}, sub { mockResponse(HTTP_CREATED, <<'EOL'); });
{
    "authorizationId" : "S02-3029547-9718378",
    "authorizationReferenceId" : "Sample-ref-id-max-32-char",
    "creationTimestamp" : "2014-07-31T16:41:00Z",
    "authorizationStatus" : {
        "state" : "Open"
    },
    "authorizationAmount" : {
        "currency" : "EUR",
        "amount" : "99.99"
    }
}
EOL

    my $Client = $self->restClient();

    # expect to be ok.
    my $Authorization = $Client->authorize('refId', 'authRefId', '1.23', 'EUR');
    is($Authorization->authorizationId, 'S02-3029547-9718378', 'successful authorize');
}

sub testGetAuthorizationDetails {
    my $self = shift;

    $mock_ua->unmap_all;
    $mock_ua->map(qr{/rs/payments/amazon/authorizations/authRefId}, sub { mockResponse(HTTP_OK, <<'EOL'); });
{
    "authorizationId" : "S02-3029547-9718378",
    "authorizationReferenceId" : "Sample-ref-id-max-32-char",
    "creationTimestamp" : "2014-07-31T16:41:00Z",
    "authorizationStatus" : {
        "state" : "Closed"
    },
    "authorizationAmount" : {
        "currency" : "EUR",
        "amount" : "99.99"
    }
}
EOL

    my $Client = $self->restClient();
    my $Authorization = $Client->getAuthorizationDetails('authRefId');
    is($Authorization->authorizationId, 'S02-3029547-9718378', 'successful getAuthorizationDetails');
    is($Authorization->authorizationStatus, 'Closed', 'successful getAuthorizationDetails');
}

sub testCapture {
    my $self = shift;

    $mock_ua->unmap_all;
    $mock_ua->map(qr{/rs/payments/amazon/capture}, sub { mockResponse(HTTP_CREATED, <<'EOL'); });
{
    "captureId" : "S02-3029547-9718378",
    "captureReferenceId" : "captureRefId",
    "creationTimestamp" : "2014-07-31T16:41:00Z",
    "captureStatus" : {
        "state" : "Pending"
    },
    "captureAmount" : {
        "currency" : "EUR",
        "amount" : "1.23"
    }
}
EOL

    my $Client = $self->restClient();

    # expect to be ok.
    # $authId, $captureRefId, $currency, $amount
    my $Capture = $Client->capture('authId', 'captureRefId', '1.23', 'EUR');
    is($Capture->captureId, 'S02-3029547-9718378', 'successful capture');
}

sub testCloseAuthorization {
    my $self = shift;

    $mock_ua->unmap_all;
    $mock_ua->map(qr{/rs/payments/amazon/authorizations/authId/close}, sub { mockResponse(HTTP_NO_CONTENT, undef); });
    my $Client = $self->restClient();

    # expect to be ok.
    # $authId, $captureRefId, $currency, $amount
    $Client->closeAuthorization('authId');
    pass('closeAuthorization ok');
}

sub testGetCaptureDetails {
    my $self = shift;

    $mock_ua->unmap_all;
    $mock_ua->map(qr{/rs/payments/amazon/capture/captureId}, sub { mockResponse(HTTP_OK, <<'EOL'); });
{
    "captureId" : "captureId",
    "captureReferenceId" : "captureRefId",
    "creationTimestamp" : "2014-07-31T16:41:00Z",
    "captureStatus" : {
        "state" : "Pending"
    },
    "captureAmount" : {
        "currency" : "EUR",
        "amount" : "1.23"
    }
}
EOL

    my $Client = $self->restClient();

    # expect to be ok.
    # $authId, $captureRefId, $currency, $amount
    my $Capture = $Client->getCaptureDetails('captureId');
    is($Capture->captureId, 'captureId', 'successful getCaptureDetails');
}

########### test helper ###############
sub restClient {
    my $self = shift;
    my (%Args) = @_;
    return DE_EPAGES::AmazonPayments::API::Object::AmazonRestClient->create($self->paymentMethod, {
        BaseURL => 'http://baseUrl/rs/',
        isLogging => 0,
        Region => 'DE',
        hKeySet => {},
        %Args
    });
}

sub paymentMethod {
    my $self = shift;
    my $pm = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments', {
        Alias => 'AmazonTest',
        IsInTestMode => 1,
        OperationMode => AMAZON_SANDBOX_MODE,
    });
    $pm->mock('getSite', sub {
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            Alias => 'AmazonTest',
            GUID  => 'guid'
        });
    });
    return $pm;
}

__PACKAGE__->runScript;
